--[[

Copyright (C) 2004 by neuron
aagaande AT gmail DOT com

This program is free software; you can redistribute it and--or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

]]

local nQuestLog = nQuestLog

local REVISION = tonumber(("$Revision: 55917 $"):match("%d+"))
if ( nQuestLog.revision < REVISION ) then
	nQuestLog.version = (nQuestLog.versionstring):format(REVISION)
	nQuestLog.revision = REVISION
	nQuestLog.date = ("$Date: 2007-11-24 18:09:14 -0500 (Sat, 24 Nov 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local moduleName = "Notifications"
local Notification = nQuestLog:NewModule(moduleName, "AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1", "Sink-1.0")

--Load libraries
local Quixote = AceLibrary("Quixote-1.0")

--Load localization
local L = AceLibrary("AceLocale-2.2"):new("Notifications")

function Notification:OnModuleLoad()
	self.menuName = L["Notifications"]
	self.description = L["Description"]
	if ( not self.defaultsRegistered ) then
		self.core:RegisterDefaults(moduleName, "profile", { enabled = true,
			notifyFinishedQuest = true, notifyFinishedQuestSound = true,
			notifyFinishedObjective = true, notifyLootedQuestItem = true,
			notifyLootedHiddenQuestItemOnly = false, objAddQuestName = false,
			hideBlizz = true, sink10OutputSink = "UIErrorsFrame" })
		self.defaultsRegistered = true
	end
end

function Notification:ToggleEnabled(v)
	self.db.profile.enabled = v
	if ( v ) then
		self:Hook("UIErrorsFrame_OnEvent", true)
		self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Objective_Update")
		self:RegisterEvent("Quixote_Quest_Complete", "OnQuest_Complete")
	else
		self:Unhook("UIErrorsFrame_OnEvent", true)
		self:UnregisterEvent("Quixote_Leaderboard_Update", "OnQuest_Objective_Update")
		self:UnregisterEvent("Quixote_Quest_Complete", "OnQuest_Complete")
	end
end

function Notification:GetMenu(oldMenu)
	local menu = { order = oldMenu.order, type = "group",
		name = self.menuName, desc = self.description,
		cmdHidden = true,
		args = {
			toggleEnabled = oldMenu,
			spacer1 = { order = 1300, type = "header" },
			testOutput = { order = 1302, type = "execute",
				name = L["testOutput"], desc = L["testOutputDesc"],
				func = function() self:Notify(L["fakeQuest"], true) end },
			spacer2 = { order = 1303, type = "header" },
			questDone = { order = 1304, type = "toggle",
				name = L["questDone"], desc = L["questDoneDesc"],
				get = function() return self.db.profile.notifyFinishedQuest end,
				set = function(v) self.db.profile.notifyFinishedQuest = v end },
			questSound = { order = 1305, type = "toggle",
				name = " - "..L["questSound"], desc = L["questSoundDesc"],
				disabled = function() return not self.db.profile.notifyFinishedQuest end,
				get = function() return self.db.profile.notifyFinishedQuestSound end,
				set = function(v) self.db.profile.notifyFinishedQuestSound = v end },
			spacer3 = { order = 1306, type = "header" },
			objDone = { order = 1307, type = "toggle",
				name = L["objDone"], desc = L["objDoneDesc"],
				get = function() return self.db.profile.notifyFinishedObjective end,
				set = function(v) self.db.profile.notifyFinishedObjective = v end },
			questItem = { order = 1308, type = "toggle",
				name = L["questItem"], desc = L["questItemDesc"],
				disabled = function() return self.db.profile.notifyLootedHiddenQuestItemOnly end,
				get = function() return self.db.profile.notifyLootedQuestItem end,
				set = function(v) self.db.profile.notifyLootedQuestItem = v end },
			hiddenQuestItem = { order = 1309, type = "toggle",
				name = " - "..L["hiddenQuestItem"], desc = L["hiddenQuestItemDesc"],
				disabled = function() return not self.db.profile.notifyLootedQuestItem end,
				get = function() return self.db.profile.notifyLootedHiddenQuestItemOnly end,
				set = function(v) self.db.profile.notifyLootedHiddenQuestItemOnly = v end },
			objQuest = { order = 1310, type = "toggle",
				name = L["objQuest"], desc = L["objQuestDesc"],
				get = function() return self.db.profile.objAddQuestName end,
				set = function(v) self.db.profile.objAddQuestName = v end },
			hideBlizz = { order = 1311, type = "toggle",
				name = L["hideBlizz"], desc = L["hideBlizzDesc"],
				get = function() return self.db.profile.hideBlizz end,
				set = function(v) self.db.profile.hideBlizz = v end } } }
	
	--Add SinkLib options menu
	menu.args.output = AceLibrary("Sink-1.0"):GetAceOptionsDataTable(self).output
	
	--Set SinkLib menu order
	menu.args.output.order = 1301
	
	return menu
end

function Notification:Notify(text, finished)
	--If quest is finished and quest completion sounds are enabled
	if ( finished ) and ( self.db.profile.notifyFinishedQuestSound ) then
		PlaySoundFile("Sound\\Creature\\Peon\\PeonBuildingComplete1.wav")
	end
	
	local color = self.core:GetColor("notificationColor")
	--Send output to SinkLib
	self:Pour(text, color.r / 255, color.g / 255, color.b / 255)
end

--Copied from FuBar_QuestsFu
function Notification:UIErrorsFrame_OnEvent(event, message)
	--If the option is enabled, then suppress Blizzard objective progress messages
	if ( self.db.profile.hideBlizz ) and ( event == "UI_INFO_MESSAGE" ) and ( message:match("^.+: ?%d+/%d+$") ) then
		return
	end
	return self.hooks.UIErrorsFrame_OnEvent(event, message)
end

function Notification:OnQuest_Objective_Update(title, id, _, desc, _, got, needed)
	if ( self.db.profile.objAddQuestName ) then
		desc = "("..title..") "..desc
	end
	--If objective is complete and notifications should be displayed for completed objectives
	if ( got == needed ) and ( self.db.profile.notifyFinishedObjective ) then
		local objs, objsDone = Quixote:GetNumQuestObjectives(id)
		--If quest isn't complete or if quest completion notifications aren't enabled
		if not (( self.db.profile.notifyFinishedQuest ) and ( objs == objsDone )) then
			self:Notify(L["objDone"]..":  "..desc)
		end
	--If there is objective progress and notifications should be displayed for objective progress
	elseif ( got > 0 ) and ( self.db.profile.notifyLootedQuestItem ) then
		local zone = select(7, Quixote:GetQuestByName(title))
		--If notifications are not restricted to hidden quests only, send a notification
		--Or, if they are, check if the quest or zone is hidden and send notification if it is
		if ( not self.db.profile.notifyLootedHiddenQuestItemOnly ) or (( self.db.profile.notifyLootedHiddenQuestItemOnly )
		and (( not self.core.db.char.watchedQuests[title] ) or ( self.core.db.char.hiddenZones[zone] ))) then
			self:Notify(desc.." "..got.."/"..needed)
		end
	end
end

function Notification:OnQuest_Complete(title)
	--If quest completion notifications are enabled
	if ( self.db.profile.notifyFinishedQuest ) then
		self:Notify(L["questDone"]..":  "..title, true)
	end
end